@@ -5,21 +5,22 @@
 #'    The function automatically manages data pre-processing, reservoir
 #'    generation (i.e., internal states) and model estimation and selection.
 #' 
-#' @param y Numeric vector containing the response variable.
+#' @param y Numeric vector containing the response variable (no missing values).
 #' @param lags Integer vector with the lag(s) associated with the input variable.
-#' @param inf_crit Character value. The information criterion used for variable selection \code{inf_crit = c("aic", "aicc", "bic", "hqc")}.
-#' @param n_diff Integer vector. The nth-differences of the response variable.
-#' @param n_models Integer value. The maximum number of (random) models to train for model selection.
-#' @param n_states Integer value. The number of internal states per reservoir.
-#' @param n_initial Integer value. The number of observations of internal states for initial drop out (throw-off).
+#' @param inf_crit Character value. Information criterion used for model selection among the \code{n_models} candidate ridge fits (different random \code{lambda} values). The candidate with the smallest criterion value is selected. One of \code{c("aic", "aicc", "bic", "hqc")}.
+#' @param n_diff Integer value. The nth-differences of the response variable. If \code{n_diff = NULL}, the number of differences required to achieve stationarity is determined automatically via a KPSS-test.
+#' @param n_models Integer value. The maximum number of (random) models to train for model selection. If \code{n_models = NULL}, the number of models is defined as \code{n_states*2}.
+#' @param n_states Integer value. The number of internal states of the reservoir. If \code{n_states = NULL}, the reservoir size is determined by \code{min(floor(n_total * tau), 200)}, where \code{n_total} is the time series length.
+#' @param n_initial Integer value. The number of observations of internal states for initial drop out (throw-off). If \code{n_initial = NULL}, the throw-off is defined as \code{n_total*0.05}, where \code{n_total} is the time series length.
 #' @param n_seed Integer value. The seed for the random number generator (for reproducibility).
-#' @param alpha Numeric value. The leakage rate (smoothing parameter) applied to the reservoir.
-#' @param rho Numeric value. The spectral radius for scaling the reservoir weight matrix.
-#' @param density Numeric value. The connectivity of the reservoir weight matrix (dense or sparse).
-#' @param lambda Numeric vector. Lower and upper bound of lambda sequence for ridge regression.
-#' @param scale_win Numeric value. The lower and upper bound of the uniform distribution for scaling the input weight matrix.
-#' @param scale_wres Numeric value. The lower and upper bound of the uniform distribution for scaling the reservoir weight matrix.
-#' @param scale_inputs Numeric vector. The lower and upper bound for scaling the time series data.
+#' @param alpha Numeric value. The leakage rate (smoothing parameter) applied to the reservoir (value greater than 0 and less than or equal to 1).
+#' @param rho Numeric value. The spectral radius for scaling the reservoir weight matrix (value often between 0 and 1, but values above 1 are possible).
+#' @param tau Numeric value. The reservoir scaling parameter to determine the reservoir size based on the time series length (value greater than 0 and less than or equal to 1).
+#' @param density Numeric value. The connectivity of the reservoir weight matrix (dense or sparse) (value greater than 0 and less than or equal to 1).
+#' @param lambda Numeric vector. Lower and upper bound of lambda range for ridge regression (numeric vector of length 2 with both values greater than 0 and \code{lambda[1]} < \code{lambda[2]}).
+#' @param scale_win Numeric value. The lower and upper bound of the uniform distribution for scaling the input weight matrix (value greater than 0, weights are sampled from U(-\code{scale_win}, \code{scale_win})).
+#' @param scale_wres Numeric value. The lower and upper bound of the uniform distribution for scaling the reservoir weight matrix (value greater than 0, weights are sampled from U(-\code{scale_wres}, \code{scale_wres}) before applying \code{rho} and \code{density}).
+#' @param scale_inputs Numeric vector. The lower and upper bound for scaling the time series data (numeric vector of length 2 with \code{scale_inputs[1]} < \code{scale_inputs[2]}, often symmetric, e.g., \code{c(-0.5, 0.5)} or \code{c(-1, 1)}).
 #' 
 #' @return A \code{list} containing:
 #'    \itemize{
@@ -29,10 +30,23 @@
 #'       \item{\code{states_train}: Numeric matrix containing the internal states.}
 #'       \item{\code{method}: A \code{list} containing several objects and meta information of the trained ESN (weight matrices, hyperparameters, model metrics, etc.).}
 #'       }
+#'  
+#' @family base functions
+#' 
+#' @references 
+#'    \itemize{
+#'       \item{Häußer, A. (2026). Echo State Networks for Time Series Forecasting: Hyperparameter Sweep and Benchmarking. arXiv preprint arXiv:2602.03912, 2026. \url{https://arxiv.org/abs/2602.03912}}
+#'       \item{Jaeger, H. (2001). The “echo state” approach to analysing and training recurrent neural networks with an erratum note. Bonn, Germany: German National Research Center for Information Technology GMD Technical Report, 148(34):13.}
+#'       \item{Jaeger, H. (2002). Tutorial on training recurrent neural networks, covering BPPT, RTRL, EKF and the "echo state network" approach.}
+#'       \item{Lukosevicius, M. (2012). A practical guide to applying echo state networks. In Neural Networks: Tricks of the Trade: Second Edition, pages 659–686. Springer.}
+#'       \item{Lukosevicius, M. and Jaeger, H. (2009). Reservoir computing approaches to recurrent neural network training. Computer Science Review, 3(3):127–149.}
+#'    }
+#' 
 #' @examples
 #' xdata <- as.numeric(AirPassengers)
 #' xmodel <- train_esn(y = xdata)
 #' summary(xmodel)
+#' 
 #' @export
 
 train_esn <- function(y,
@@ -45,6 +59,7 @@ train_esn <- function(y,
                       n_seed = 42,
                       alpha = 1,
                       rho = 1,
+                      tau = 0.4,
                       density = 0.5,
                       lambda = c(1e-4, 2),
                       scale_win = 0.5,
@@ -58,18 +73,18 @@ train_esn <- function(y,
   if (is.vector(y) & is.numeric(y)) {
     n_outputs <- 1
   } else {
-    stop("train_esn() requires a numeric vector as input.")
+    stop("`train_esn()` requires a numeric vector as input.")
   }
   
   if(any(is.na(y))){
-    stop("train_esn() does not support missing values.")
+    stop("`train_esn()` does not support missing values.")
   }
   
   # Number of observations
   n_total <- length(y)
   
   if (is.null(n_states)) {
-    n_states <- min(floor(n_total * 0.4), 200)
+    n_states <- min(floor(n_total * tau), 200)
   }
   
   if (is.null(n_models)) {
@@ -77,13 +92,26 @@ train_esn <- function(y,
   }
   
   # Number of initial observations to drop
-  n_initial <- floor(n_total * 0.05)
-  
+  if (is.null(n_initial)) {
+    n_initial <- floor(n_total * 0.05)
+  }
+
   # Number of differences required to achieve stationarity
   if (is.null(n_diff)) {
     n_diff <- estimate_ndiff(y)
   }
   
+  # Check information criterion
+  inf_crit <- match.arg(
+    inf_crit, 
+    choices = c("aic", "aicc", "bic", "hqc")
+    )
+  
+  # Check time series length
+  if (n_total - n_diff - max(lags) - n_initial <= 0) {
+    stop("`y` is too short for given `n_diff`, `lags`, and `n_initial`.")
+  }
+  
   # Set seed for reproducibility
   set.seed(n_seed)
   
@@ -210,10 +238,6 @@ train_esn <- function(y,
       .before = "loglik") %>%
     arrange(!!sym(inf_crit))
   
-  # Alternative base R code
-  # model_metrics <- cbind(model = model_names, model_metrics)
-  # model_metrics <- model_metrics[ order(model_metrics[[inf_crit]]) , ]
-  
   # Identify best model, lambda and degrees of freedom (extract first row)
   model_best <- model_metrics[["model"]][1]
   lambda <- model_metrics[["lambda"]][1]
